@@ -15,6 +15,7 @@
 #include "nest/cli.h"
 #include "nest/iface.h"
 #include "filter/filter.h"
+#include "sysdep/unix/krt.h"
 
 static void
 rt_show_table(struct cli *c, struct rt_show_data *d)
@@ -28,14 +29,20 @@ rt_show_table(struct cli *c, struct rt_show_data *d)
   d->last_table = d->tab;
 }
 
+static inline struct krt_proto *
+rt_show_get_kernel(struct rt_show_data *d)
+{
+  struct proto_config *krt = d->tab->table->config->krt_attached;
+  return krt ? (struct krt_proto *) krt->proto : NULL;
+}
+
 static void
-rt_show_rte(struct cli *c, byte *ia, rte *e, struct rt_show_data *d)
+rt_show_rte(struct cli *c, byte *ia, rte *e, struct rt_show_data *d, int primary)
 {
   byte from[IPA_MAX_TEXT_LENGTH+8];
   byte tm[TM_DATETIME_BUFFER_SIZE], info[256];
   rta *a = e->attrs;
-  int primary = (e->net->routes == e);
-  int sync_error = (e->net->n.flags & KRF_SYNC_ERROR);
+  int sync_error = d->kernel ? krt_get_sync_error(d->kernel, e) : 0;
   void (*get_route_info)(struct rte *, byte *buf);
   struct nexthop *nh;
 
@@ -97,6 +104,11 @@ rt_show_net(struct cli *c, net *n, struct rt_show_data *d)
   rte *e, *ee;
   byte ia[NET_MAX_TEXT_LENGTH+1];
   struct channel *ec = d->tab->export_channel;
+
+  /* The Clang static analyzer complains that ec may be NULL.
+   * It should be ensured to be not NULL by rt_show_prepare_tables() */
+  ASSUME(!d->export_mode || ec);
+
   int first = 1;
   int pass = 0;
 
@@ -121,9 +133,17 @@ rt_show_net(struct cli *c, net *n, struct rt_show_data *d)
       if (ec && (ec->export_state == ES_DOWN))
 	goto skip;
 
-      /* Special case for merged export */
-      if ((d->export_mode == RSEM_EXPORT) && (ec->ra_mode == RA_MERGED))
+      if (d->export_mode == RSEM_EXPORTED)
+        {
+	  if (!bmap_test(&ec->export_map, ee->id))
+	    goto skip;
+
+	  // if (ec->ra_mode != RA_ANY)
+	  //   pass = 1;
+        }
+      else if ((d->export_mode == RSEM_EXPORT) && (ec->ra_mode == RA_MERGED))
 	{
+	  /* Special case for merged export */
 	  rte *rt_free;
 	  e = rt_export_merged(ec, n, &rt_free, c->show_pool, 1);
 	  pass = 1;
@@ -167,7 +187,7 @@ rt_show_net(struct cli *c, net *n, struct rt_show_data *d)
 	goto skip;
 
       if (d->stats < 2)
-	rt_show_rte(c, ia, e, d);
+	rt_show_rte(c, ia, e, d, (e->net->routes == ee));
 
       d->show_counter++;
       ia[0] = 0;
@@ -223,6 +243,7 @@ rt_show_cont(struct cli *c)
     FIB_ITERATE_INIT(&d->fit, &d->tab->table->fib);
     d->table_open = 1;
     d->table_counter++;
+    d->kernel = rt_show_get_kernel(d);
 
     d->show_counter_last = d->show_counter;
     d->rt_counter_last   = d->rt_counter;
@@ -253,6 +274,7 @@ rt_show_cont(struct cli *c)
 	       d->net_counter - d->net_counter_last, d->tab->table->name);
   }
 
+  d->kernel = NULL;
   d->table_open = 0;
   d->tab = NODE_NEXT(d->tab);
 
@@ -396,6 +418,7 @@ rt_show(struct rt_show_data *d)
     WALK_LIST(tab, d->tables)
     {
       d->tab = tab;
+      d->kernel = rt_show_get_kernel(d);
 
       if (d->show_for)
 	n = net_route(tab->table, d->addr);
