@@ -329,6 +329,14 @@ ospf_originate_lsa(struct ospf_proto *p, struct ospf_new_lsa *lsa)
     en->next_lsa_opts = 0;
   }
 
+  /* The static analyzer complains here that en->lsa_body may be NULL.
+   * Yes, it may if ospf_hash_get() creates a new struct top_hash_entry.
+   * In this case, also en->lsa.length must be 0 and lsa_length is never
+   * equal to 0 while sizeof(struct ospf_lsa_header) is non-zero.
+   * Therefore memcmp() is never executed with NULL here.
+   * */
+  ASSUME(en->lsa.age >= LSA_MAXAGE || (en->lsa.length == 0) == (en->lsa_body == NULL));
+
   /* Ignore the the new LSA if is the same as the current one */
   if ((en->lsa.age < LSA_MAXAGE) &&
       (lsa_length == en->lsa.length) &&
@@ -789,14 +797,11 @@ prepare_rt2_lsa_body(struct ospf_proto *p, struct ospf_area *oa)
 	if (neigh->state == NEIGHBOR_FULL)
 	{
 	  /*
-	   * ln->data should be ifa->iface_id in case of no/ptp
-	   * address (ifa->addr->flags & IA_PEER) on PTP link (see
-	   * RFC 2328 12.4.1.1.), but the iface ID value has no use,
-	   * while using IP address even in this case is here for
-	   * compatibility with some broken implementations that use
-	   * this address as a next-hop.
+	   * ln->data field should be ifa->iface_id for unnumbered PtP links,
+	   * IP address otherwise (see RFC 2328 12.4.1.1). This is controlled
+	   * by ifa->ptp_address field.
 	   */
-	  add_rt2_lsa_link(p, LSART_PTP, neigh->rid, ipa_to_u32(ifa->addr->ip), link_cost);
+	  add_rt2_lsa_link(p, LSART_PTP, neigh->rid, ospf_iface_get_data(ifa), link_cost);
 	  i++;
 	}
       break;
@@ -1450,6 +1455,9 @@ ospf_originate_link_lsa(struct ospf_proto *p, struct ospf_iface *ifa)
   prepare_link_lsa_body(p, ifa);
 
   ifa->link_lsa = ospf_originate_lsa(p, &lsa);
+
+  /* RFC 5340 4.4.3 Events 6+7 - new Link LSA is locally originated */
+  ospf_notify_net_lsa(ifa);
 }
 
 
@@ -2100,9 +2108,7 @@ ospf_hash_get(struct top_graph *f, u32 domain, u32 lsa, u32 rtr, u32 type)
   if (e)
     return e;
 
-  e = sl_alloc(f->hash_slab);
-  bzero(e, sizeof(struct top_hash_entry));
-
+  e = sl_allocz(f->hash_slab);
   e->color = OUTSPF;
   e->dist = LSINFINITY;
   e->lsa.type_raw = type;
