@@ -141,6 +141,8 @@ pipe_postconfig(struct proto_config *CF)
 
   if (cc->in_keep_filtered)
     cf_error("Pipe protocol prohibits keeping filtered routes");
+
+  cc->debug = cf->c.debug;
 }
 
 static int
@@ -154,7 +156,9 @@ pipe_configure_channels(struct pipe_proto *p, struct pipe_config *cf)
     .table = cc->table,
     .out_filter = cc->out_filter,
     .in_limit = cc->in_limit,
-    .ra_mode = RA_ANY
+    .ra_mode = RA_ANY,
+    .debug = cc->debug,
+    .rpki_reload = cc->rpki_reload,
   };
 
   struct channel_config sec_cf = {
@@ -163,7 +167,9 @@ pipe_configure_channels(struct pipe_proto *p, struct pipe_config *cf)
     .table = cf->peer,
     .out_filter = cc->in_filter,
     .in_limit = cc->out_limit,
-    .ra_mode = RA_ANY
+    .ra_mode = RA_ANY,
+    .debug = cc->debug,
+    .rpki_reload = cc->rpki_reload,
   };
 
   return
@@ -276,6 +282,14 @@ pipe_show_proto_info(struct proto *P)
     pipe_show_stats(p);
 }
 
+void
+pipe_update_debug(struct proto *P)
+{
+  struct pipe_proto *p = (void *) P;
+
+  p->pri->debug = p->sec->debug = p->p.debug;
+}
+
 
 struct protocol proto_pipe = {
   .name =		"Pipe",
