@@ -729,12 +729,22 @@ rpki_prefix_pdu_2_net_addr(const struct pdu_header *pdu, net_addr_union *n)
 static int
 rpki_handle_prefix_pdu(struct rpki_cache *cache, const struct pdu_header *pdu)
 {
+  const struct rpki_config *cf = (void *) cache->p->p.cf;
+
   const enum pdu_type type = pdu->type;
   ASSERT(type == IPV4_PREFIX || type == IPV6_PREFIX);
 
   net_addr_union addr = {};
   rpki_prefix_pdu_2_net_addr(pdu, &addr);
 
+  if (cf->ignore_max_length)
+  {
+    if (type == IPV4_PREFIX)
+      addr.roa4.max_pxlen = IP4_MAX_PREFIX_LENGTH;
+    else
+      addr.roa6.max_pxlen = IP6_MAX_PREFIX_LENGTH;
+  }
+
   struct channel *channel = NULL;
 
   if (type == IPV4_PREFIX)
@@ -1011,6 +1021,7 @@ rpki_send_error_pdu(struct rpki_cache *cache, const enum pdu_error_type error_co
   {
     va_start(args, fmt);
     msg_len = bvsnprintf(msg, sizeof(msg), fmt, args) + 1;
+    va_end(args);
   }
 
   u32 pdu_size = 16 + err_pdu_len + msg_len;
